@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugins-engine.c 6727 2008-12-25 21:02:48Z pborelli $
+ * $Id: gedit-plugins-engine.c 6790 2009-01-01 22:47:29Z jessevdk $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -630,6 +630,19 @@ gedit_plugins_engine_activate_plugin (GeditPluginsEngine *engine,
 	return gedit_plugin_info_is_active (info);
 }
 
+static void
+call_plugin_deactivate (GeditPlugin *plugin, 
+			GeditWindow *window)
+{
+	gedit_plugin_deactivate (plugin, window);
+
+	/* ensure update of ui manager, because we suspect it does something
+	   with expected static strings in the type module (when unloaded the
+	   strings don't exist anymore, and ui manager updates in an idle
+	   func) */
+	gtk_ui_manager_ensure_update (gedit_window_get_ui_manager (window));
+}
+
 static void
 gedit_plugins_engine_deactivate_plugin_real (GeditPluginsEngine *engine,
 					     GeditPluginInfo *info)
@@ -643,7 +656,7 @@ gedit_plugins_engine_deactivate_plugin_real (GeditPluginsEngine *engine,
 
 	wins = gedit_app_get_windows (gedit_app_get_default ());
 	for (; wins != NULL; wins = wins->next)
-		gedit_plugin_deactivate (info->plugin, GEDIT_WINDOW (wins->data));
+		call_plugin_deactivate (info->plugin, GEDIT_WINDOW (wins->data));
 
 	/* first unref the plugin (the loader still has one) */
 	g_object_unref (info->plugin);
