@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugins-engine.h 6264 2008-05-05 11:00:38Z sfre $
+ * $Id: gedit-plugins-engine.h 6727 2008-12-25 21:02:48Z pborelli $
  */
 
 #ifndef __GEDIT_PLUGINS_ENGINE_H__
@@ -78,6 +78,7 @@ const GList	*gedit_plugins_engine_get_plugin_list 	(GeditPluginsEngine *engine);
 GeditPluginInfo	*gedit_plugins_engine_get_plugin_info	(GeditPluginsEngine *engine,
 							 const gchar        *name);
 
+/* plugin load and unloading (overall, for all windows) */
 gboolean 	 gedit_plugins_engine_activate_plugin 	(GeditPluginsEngine *engine,
 							 GeditPluginInfo    *info);
 gboolean 	 gedit_plugins_engine_deactivate_plugin	(GeditPluginsEngine *engine,
@@ -87,13 +88,17 @@ void	 	 gedit_plugins_engine_configure_plugin	(GeditPluginsEngine *engine,
 							 GeditPluginInfo    *info,
 							 GtkWindow          *parent);
 
-/* 
- * new_window == TRUE if this function is called because a new top window
- * has been created
- */
-void		 gedit_plugins_engine_update_plugins_ui (GeditPluginsEngine *engine,
-							 GeditWindow        *window, 
-							 gboolean            new_window);
+/* plugin activation/deactivation per window, private to GeditWindow */
+void 		 gedit_plugins_engine_activate_plugins   (GeditPluginsEngine *engine,
+							  GeditWindow        *window);
+void 		 gedit_plugins_engine_deactivate_plugins (GeditPluginsEngine *engine,
+							  GeditWindow        *window);
+void		 gedit_plugins_engine_update_plugins_ui  (GeditPluginsEngine *engine,
+							  GeditWindow        *window);
+
+/* private for gconf notification */
+void		 gedit_plugins_engine_active_plugins_changed
+							(GeditPluginsEngine *engine);
 
 G_END_DECLS
 
