@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c 6606 2008-11-22 15:02:10Z icq $ 
+ * $Id: gedit-view.c 6809 2009-01-05 18:04:39Z pborelli $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +43,7 @@
 
 #include "gedit-view.h"
 #include "gedit-debug.h"
+#include "gedit-prefs-manager.h"
 #include "gedit-prefs-manager-app.h"
 #include "gedit-marshal.h"
 #include "gedit-utils.h"
@@ -123,6 +124,8 @@ static gboolean gedit_view_drag_drop		(GtkWidget        *widget,
 	      					 gint              x,
 	      					 gint              y,
 	      					 guint             time);
+static gboolean	gedit_view_button_press_event	(GtkWidget        *widget,
+						 GdkEventButton   *event);
 
 static gboolean start_interactive_search	(GeditView        *view);
 static gboolean start_interactive_goto_line	(GeditView        *view);
@@ -207,7 +210,7 @@ gedit_view_class_init (GeditViewClass *klass)
 	widget_class->drag_motion = gedit_view_drag_motion;
 	widget_class->drag_data_received = gedit_view_drag_data_received;
 	widget_class->drag_drop = gedit_view_drag_drop;
-
+	widget_class->button_press_event = gedit_view_button_press_event;
 	klass->start_interactive_search = start_interactive_search;
 	klass->start_interactive_goto_line = start_interactive_goto_line;
 	klass->reset_searched_text = reset_searched_text;	
@@ -580,6 +583,13 @@ gedit_view_paste_clipboard (GeditView *view)
 				      0.0);
 }
 
+/**
+ * gedit_view_delete_selection:
+ * @view: a #GeditView
+ * 
+ * Deletes the text currently selected in the #GtkTextBuffer associated
+ * to the view and scroll to the cursor position.
+ **/
 void
 gedit_view_delete_selection (GeditView *view)
 {
@@ -606,6 +616,12 @@ gedit_view_delete_selection (GeditView *view)
 				      0.0);
 }
 
+/**
+ * gedit_view_select_all:
+ * @view: a #GeditView
+ * 
+ * Selects all the text displayed in the @view.
+ **/
 void
 gedit_view_select_all (GeditView *view)
 {
@@ -623,6 +639,12 @@ gedit_view_select_all (GeditView *view)
 	gtk_text_buffer_select_range (buffer, &start, &end);
 }
 
+/**
+ * gedit_view_scroll_to_cursor:
+ * @view: a #GeditView
+ * 
+ * Scrolls the @view to the cursor position.
+ **/
 void
 gedit_view_scroll_to_cursor (GeditView *view)
 {
@@ -643,6 +665,15 @@ gedit_view_scroll_to_cursor (GeditView *view)
 				      0.0);
 }
 
+/**
+ * gedit_view_set_font:
+ * @view: a #GeditView
+ * @gboolean: whether to reset the default font
+ * @font_name: the name of the font to use
+ * 
+ * If @def is #TRUE, resets the font of the @view to the default font
+ * otherwise sets it to @font_name.
+ **/
 void
 gedit_view_set_font (GeditView   *view, 
 		     gboolean     def, 
@@ -1910,6 +1941,70 @@ gedit_view_drag_drop (GtkWidget      *widget,
 	return result;
 }
 
+static void
+show_line_numbers_toggled (GtkMenu   *menu,
+			   GeditView *view)
+{
+	gboolean show;
+
+	show = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menu));
+
+	gedit_prefs_manager_set_display_line_numbers (show);
+}
+
+static GtkWidget *
+create_line_numbers_menu (GtkWidget *view)
+{
+	GtkWidget *menu;
+	GtkWidget *item;
+
+	menu = gtk_menu_new ();
+
+	item = gtk_check_menu_item_new_with_mnemonic (_("_Display line numbers"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+					gtk_source_view_get_show_line_numbers (GTK_SOURCE_VIEW (view)));
+	g_signal_connect (item, "toggled",
+			  G_CALLBACK (show_line_numbers_toggled), view);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	
+	gtk_widget_show_all (menu);
+	
+	return menu;
+}
+
+static void
+show_line_numbers_menu (GtkWidget      *view,
+			GdkEventButton *event)
+{
+	GtkWidget *menu;
+
+	menu = create_line_numbers_menu (view);
+
+	gtk_menu_popup (GTK_MENU (menu), 
+			NULL, 
+			NULL,
+			NULL, 
+			NULL,
+			event->button, 
+			event->time);
+}
+
+static gboolean
+gedit_view_button_press_event (GtkWidget *widget, GdkEventButton *event)
+{
+	if ((event->type == GDK_BUTTON_PRESS) && 
+	    (event->button == 3) &&
+	    (event->window == gtk_text_view_get_window (GTK_TEXT_VIEW (widget),
+						        GTK_TEXT_WINDOW_LEFT)))
+	{
+		show_line_numbers_menu (widget, event);
+
+		return TRUE;
+	}
+
+	return GTK_WIDGET_CLASS (gedit_view_parent_class)->button_press_event (widget, event);
+}
+
 static void 	
 search_highlight_updated_cb (GeditDocument *doc,
 			     GtkTextIter   *start,
